package com.smith9.geiger.client.beans

import java.beans.PropertyChangeListener
import java.beans.PropertyChangeEvent

/**
 * Created: 22/03/12
 * @author david
 */
class CountList {
    private def counts = [:] as ObservableMap

    public void addSizeTrigger(int threshold, Closure sizeReachedAction) {
        counts.addPropertyChangeListener(ObservableMap.SIZE_PROPERTY, new SizeChangeListener(this, threshold, sizeReachedAction))
    }

    public List<Count> getCountValues() {
        return Collections.unmodifiableList(counts.values() as List<Count>)
    }
    public boolean isEmpty() {
        return counts.isEmpty()
    }

    public void clear() {
        counts.clear()
    }

    public int size() {
        return counts.size()
    }

    public Count remove(Count count) {
        level(count)
        return counts.remove(count.key)
    }

    public void addAll(List<Count> counts) {
        counts.each { Count count ->
            add(count)
        }
    }

    public void add(Count count) {
        level(count)
        counts[count.key] = count
    }

    private void level(Count count) {
        if (!count.isLevelled) {
            Count.levelDate(count)
        }
    }

    public void merge(Count count) {
        Count existing = find(count)
        if (existing) {
            existing.merge(count)
        } else {
//            Count shallowCopy = count.createCopyLevelDate()
//            Count.levelDate(shallowCopy)
            add(count)
        }
    }

    public Count find(Count count) {
        level(count)
        return counts[count.key]
    }
}

private class SizeChangeListener implements PropertyChangeListener {
    private Closure sizeChangeAction
    private int sizeThreshold
    private CountList counts

    SizeChangeListener(CountList counts, int sizeThreshold, Closure sizeChangeAction) {
        this.sizeChangeAction = sizeChangeAction
        this.sizeThreshold = sizeThreshold
        this.counts = counts
    }

    public void propertyChange(PropertyChangeEvent evt) {
        if(evt.newValue >= sizeThreshold) {
            sizeChangeAction(counts)
        }
    }
}
